"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const EveryTime_1 = __importDefault(require("./EveryTime"));
const helpers_1 = __importDefault(require("./helpers"));
function assertValidCron(cron) {
    if (cron.length !== 5)
        throw new Error('Invalid cron, expected 5 elements got ' + JSON.stringify(cron));
    return true;
}
class Task {
    name;
    executor;
    constructor(name, executor) {
        this.name = name;
        this.executor = executor;
    }
    getTimeBuilder() {
        return new Builder(this);
    }
    getName() {
        return this.name;
    }
    getExecutor() {
        return this.executor;
    }
}
class Builder {
    cron = ['0', '0', '*', '*', '*'];
    runsAt = null;
    task;
    constructor(task) {
        this.task = task;
    }
    build() {
        if (this.runsAt)
            return {
                name: this.task.getName(),
                execute: this.task.getExecutor(),
            };
    }
    replaceCron(cron) {
        const split = cron.split(' ');
        if (assertValidCron(split)) {
            this.cron = split;
        }
    }
    at(hhMmStr) {
        const [hh, mm] = hhMmStr.split(':');
        this.cron[0] = hh;
        this.cron[1] = mm;
    }
    onceAt(date) {
        this.runsAt = new Date(date);
        return {
            build: () => ({
                name: this.task.getName(),
                execute: this.task.getExecutor(),
            }),
        };
    }
    monthly() {
        this.cron[2] = '1';
        this.cron[3] = '*';
        return this;
    }
    daily() {
        this.cron[2] = '*';
        this.cron[3] = '*';
        return this;
    }
    onSpecificDays(days) {
        if (!Array.isArray(days) || days.length === 0) {
            throw new Error('onSpecificDays expects days to be an array of days string.');
        }
        const intDays = helpers_1.default.daysToIntegers(days);
        this.cron[4] = intDays.join(',');
        return this;
    }
    /**
     * Every Minute
     */
    everyMinute() {
        this.cron[0] = '*';
        this.cron[1] = '*';
        return this;
    }
    everyHour() {
        return this.everyHourAt(0);
    }
    everyHourAt(minuteOfTheHour) {
        this.cron[0] = `${minuteOfTheHour}`;
        return this;
    }
    everyWeekDay(startDay = 'monday', endDay = 'friday') {
        startDay = helpers_1.default.dayToInt(startDay);
        endDay = helpers_1.default.dayToInt(endDay);
        helpers_1.default.validateStartToEndDay(startDay, endDay);
        this.cron[4] = `${startDay}-${endDay}`;
        return this;
    }
    /**
     * Every Year
     */
    static everyYear() {
        return Builder.everyYearIn(1);
    }
    /**
     * Every Year In
     * @param monthOfTheYear  - Month of the year
     * @param dayOfTheMonth - Day of the month
     * @param hourOfTheDay - Hour of the day
     * @param minuteOfTheHour - Minute of the hour.
     */
    static everyYearIn(monthOfTheYear, dayOfTheMonth = 1, hourOfTheDay = 0, minuteOfTheHour = 0) {
        return `${minuteOfTheHour} ${hourOfTheDay} ${dayOfTheMonth} ${monthOfTheYear} *`;
    }
    /**
     * Between Time Frames
     * @param start - Start
     * @param end - End
     */
    static between(start, end) {
        return new EveryTime_1.default([start, end], {
            between: true,
        });
    }
}
exports.default = Task;
